' Media Monkey Script for MM3.0+
Option Explicit     ' report undefined variables, ...
Dim ScriptName:		ScriptName = "DB_Schema.vbs"
Dim ScriptTitle:	ScriptTitle = "Create Database Schema Report for MM3"
Dim ScriptReport:	ScriptReport = "DB_Schema_Report"
Dim ScriptAuthor:	ScriptAuthor = "Owyn"
Dim ScriptVersion:	ScriptVersion = "1.0.4"
Dim ScriptDate:		ScriptDate = "2009-10-16"

' Known problems:
' - None

' Fixes for next version:
' - None

' Revision History:
' 2009-10-16 v1.04
' -Release as MMIP
'
' 2009-01-08 v1.03
' -Rename script to DB_Schema before release
' -Cleanup script titles, etc.
' -Change to OnStartup for main
' -Add SQLite version and DBInfo to Header
'
' 2009-01-08 v1.02
' -Add SDB.VersionBuild
' -Destroy objects fso,Property on exit
'
' 2009-01-01 v1.0.1
' -Destroy qrySQL
'
' 2008-12-31 v1.0.0
' - Initial release

' Global and Configuration variables
Dim fout								' File Object

Sub OnStartup
 ' Open inifile and get last used directory
  Dim iniF
  Set iniF = SDB.IniFile

  ' Let user select the output path
  Dim path
  path = iniF.StringValue( "Scripts", "DumpSchemaDir")

  path = SDB.SelectFolder( path, SDB.Localize( "Select where to create the Schema Report."))

  If path="" Then
    Exit Sub
  End If

  If Right( path, 1)<>"\" Then
    path = path & "\"
  End If

  ' Write selected directory to the ini file
  iniF.StringValue( "Scripts", "DumpSchemaDir") = path
  Set iniF = Nothing

  ' Connect to the FileSystemObject
  Dim fso
  Set fso = SDB.Tools.FileSystem
  
  ' Get Databaase Path
  Dim strDBpath : strDBpath = SDB.Database.Path
  
  ' Get Version 
  Dim strMMVersion : strMMVersion = SDB.VersionString
  ON ERROR RESUME NEXT
  strMMVersion = strMMVersion & "." & SDB.VersionBuild
  ON ERROR GOTO 0
 
  ' Use progress to notify user about the current action
  Dim Progress, ExpText
  Set Progress = SDB.Progress
  ExpText = SDB.Localize("Creating MM3 Schema Report...")
  Progress.Text = ExpText

  Dim strTitle, strFilename
  strTitle = ScriptReport
  strFilename = path & fso.CorrectFilename( strTitle)
  strFilename = StrFilename & "(" & strMMVersion & ").txt"

  ' Create the Report File
  ' Note; fout declared globally to permit writes in Subroutines
  Set fout = fso.CreateTextFile( strFilename, True)
  
 
  ' Prepare Header for Report
  fout.WriteLine ScriptName & "(" & ScriptVersion & "):" & ScriptTitle
  fout.WriteLine "Report File: " & strFilename
  fout.Writeline "Database: " & strDBpath
  fout.Writeline "DB Info:     " & DBInfo()
  fout.Writeline "MediaMonkey: " & strMMVersion
  fout.Writeline "SQLite:      " & SQL_Version()
  fout.WriteLine "Run: " & Date
  
  '  Prepare the report
  Call GetSchema()
   
  ' Finish the Report
  fout.WriteLine "" 
  fout.WriteLine "----End of Report----"
  fout.Close
  Set fout = Nothing
  Set Progress = Nothing
  Set fso = Nothing
End Sub

Sub GetSchema ()
	Dim strSQL, qrySQL
	
	strSQL = "SELECT * FROM sqlite_master ORDER BY tbl_name, name"
	Set qrySQL = SDB.Database.OpenSQL(strSQL)
	
	Dim strSMType, strSMName, strSMTbl_Name, strSMRootPage, strSMSQL, arrFields, numFields, i
	While NOT qrySQL.EOF
		strSMType = qrySQL.StringByIndex(0)
		strSMName = qrySQL.StringByIndex(1)
		strSMTbl_Name = qrySQL.StringByIndex(2)
		strSMRootPage = qrySQL.StringByIndex(3)
		strSMSQL = qrySQL.StringByIndex(4)
		
		select case strSMType
		case "table"
			fout.Writeline " "
			arrFields = Split(strSMSQL,",")
			numFields = Ubound(arrFields, 1)
			fout.Writeline arrFields(0)
			i=1
			While i <= numFields
				fout.Writeline "    ," & Trim(arrFields(i))
				i=i+1
			Wend
		case Else
			fout.Writeline strSMSQL
		End select
		
		qrySQL.Next
	Wend
	Set qrySQL = Nothing
End Sub

' Get DBInfo
Function DBInfo ()
	Dim strSQL,qrySQL,strInfo
	strInfo=""
	strSQL = "SELECT * FROM DBInfo ORDER BY IDINFO"
	Set qrySQL = SDB.Database.OpenSQL(strSQL)
	While NOT qrySQL.EOF
		strInfo = strInfo & qrySQL.StringByIndex(1) & " "
		qrySQL.Next
	Wend
	DBInfo = strInfo
	Set qrySQL = Nothing
End Function

' Get SQLite version
Function SQL_Version ()
	Dim strSQL,qrySQL
	strSQL = "SELECT sqlite_version()"
	Set qrySQL = SDB.Database.OpenSQL(strSQL)
	SQL_Version = qrySQL.StringByIndex(0)
	Set qrySQL = Nothing
End Function
